import { types } from '@babel/core';
import { addNamed } from '@babel/helper-module-imports';

function functionDeclarationToExpression(declaration) {
  return types.functionExpression(declaration.id, declaration.params, declaration.body, declaration.generator, declaration.async);
}

function classDeclarationToExpression(declaration) {
  return types.classExpression(declaration.id, declaration.superClass, declaration.body, declaration.decorators);
}

function getFileName(path) {
  var _path$hub$file = path.hub.file,
      file = _path$hub$file === void 0 ? {} : _path$hub$file;
  var _file$opts = file.opts,
      opts = _file$opts === void 0 ? {} : _file$opts;
  var filename = opts.filename,
      cwd = opts.cwd;

  if (!filename) {
    return undefined;
  }

  if (cwd && filename.startsWith(cwd)) {
    return filename.slice(cwd.length);
  }

  return filename;
}

var functionsToReplace = ['getServerSideProps', 'getStaticProps'];

function transformPropGetters(path, transform) {
  var node = path.node;

  if (types.isFunctionDeclaration(node.declaration)) {
    var functionId = node.declaration.id;

    if (!functionId) {
      return;
    }

    if (!functionsToReplace.includes(functionId.name)) {
      return;
    }

    node.declaration = types.variableDeclaration('const', [types.variableDeclarator(functionId, transform(functionDeclarationToExpression(node.declaration)))]);
    return;
  }

  if (types.isVariableDeclaration(node.declaration)) {
    node.declaration.declarations.forEach(function (declaration) {
      if (types.isIdentifier(declaration.id) && functionsToReplace.includes(declaration.id.name) && declaration.init) {
        declaration.init = transform(declaration.init);
      }
    });
  }
}

function addWithSuperJSONPropsImport(path) {
  return addNamed(path, 'withSuperJSONProps', 'babel-plugin-superjson-next/tools');
}

function addWithSuperJSONPageImport(path) {
  return addNamed(path, 'withSuperJSONPage', 'babel-plugin-superjson-next/tools');
}

function superJsonWithNext() {
  return {
    name: 'replace gSSP',
    visitor: {
      Program: function Program(path) {
        var _getFileName;

        var filename = (_getFileName = getFileName(path)) !== null && _getFileName !== void 0 ? _getFileName : 'pages/Default.js';

        if (!filename.includes('pages/')) {
          return;
        }

        var foundGSSP = false;
        path.traverse({
          ExportNamedDeclaration: function ExportNamedDeclaration(path) {
            transformPropGetters(path, function (decl) {
              foundGSSP = true;
              return types.callExpression(addWithSuperJSONPropsImport(path), [decl]);
            });
          }
        });

        if (foundGSSP) {
          path.traverse({
            ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
              function wrapInHOC(expr) {
                return types.callExpression(addWithSuperJSONPageImport(path), [expr]);
              }

              var node = path.node;

              if (types.isIdentifier(node.declaration)) {
                node.declaration = wrapInHOC(node.declaration);
              }

              if (types.isFunctionExpression(node.declaration)) {
                node.declaration = wrapInHOC(node.declaration);
              }

              if (types.isFunctionDeclaration(node.declaration) || types.isClassDeclaration(node.declaration)) {
                if (node.declaration.id) {
                  path.insertBefore(node.declaration);
                  node.declaration = wrapInHOC(node.declaration.id);
                } else {
                  if (types.isFunctionDeclaration(node.declaration)) {
                    node.declaration = wrapInHOC(functionDeclarationToExpression(node.declaration));
                  } else {
                    node.declaration = wrapInHOC(classDeclarationToExpression(node.declaration));
                  }
                }
              }
            }
          });
        }
      }
    }
  };
}

export default superJsonWithNext;
//# sourceMappingURL=index.js.map
declaration));
                  }
                }
              }
            }
          });
        }
      }
    }
  };
}

exports.default = superJsonWithNext;
//# sourceMappingURL=index.js.map
